@@ -1,4 +1,4 @@
 library(testthat)
-expect_message(library(MortalityLaws))
+# expect_message(library(MortalityLaws))
 
 test_check("MortalityLaws")
